<?php

class Hackathon_EmailPreview_Model_EmailPreview
{
    /**
     * Returns the code of the rendered email.
     * 
     * @param int $templateId
     * @param array $templateParams
     * @param array $recipient
     * @return string
     */
    public function renderEmail($templateId, $templateParams, $recipient = false)
    {
        /**
         * @var Mage_Core_Model_App_Emulation $appEmulation
         */
        $appEmulation = Mage::getSingleton('core/app_emulation');
        $store = $templateParams['store'];
        $storeId = $store->getStoreId();
        $initialEnvironmentInfo = $appEmulation->startEnvironmentEmulation($storeId);

        /** @var Mage_Core_Model_Email_Template $template */
        $template = Mage::getModel('core/email_template');

        if (is_numeric($templateId)) {
            $template->load($templateId);
        } else {
            $localeCode = Mage::getStoreConfig('general/locale/code', $storeId);
            $template->loadDefault($templateId, $localeCode);
        }
        
        /* @var $filter Mage_Core_Model_Input_Filter_MaliciousCode */
        $filter = Mage::getSingleton('core/input_filter_maliciousCode');

        $template->setTemplateText(
            $filter->filter($template->getTemplateText())
        );

        $templateProcessed = $template->getProcessedTemplate($templateParams);

        if ($recipient) {
            $subject = $template->getProcessedTemplateSubject($templateParams);
            $this->_send($template, $subject, $templateProcessed, $recipient);
        }

        if ($template->isPlain()) {
            $templateProcessed = '<pre>' . htmlspecialchars($templateProcessed) . '</pre>';
        }
        
        // Stop store emulation process
        $appEmulation->stopEnvironmentEmulation($initialEnvironmentInfo);
        
        return $templateProcessed;
    }

    protected function _send(Mage_Core_Model_Email_Template $template, $subject, $text, $email)
    {

        $emails = array($email);
        $names = array();

        ini_set('SMTP', Mage::getStoreConfig('system/smtp/host'));
        ini_set('smtp_port', Mage::getStoreConfig('system/smtp/port'));

        $mail = $template->getMail();

        foreach ($emails as $key => $email) {
            $mail->addTo($email, '=?utf-8?B?' . base64_encode($names[$key]) . '?=');
        }

        if ($template->isPlain()) {
            $mail->setBodyText($text);
        } else {
            $mail->setBodyHtml($text);
        }

        $mail->setSubject('=?utf-8?B?' . base64_encode($subject) . '?=');
        $mail->setFrom($template->getSenderEmail(), $template->getSenderName());

        try {
            $mail->send();
        }
        catch (Exception $e) {
            Mage::logException($e);
            return false;
        }

        return true;
    }
}
